@@ -1,15 +1,17 @@
 // Adapted from https://github.com/tomusdrw/rust-web3/blob/master/src/types/log.rs
 use crate::{
-    types::{Address, BlockNumber, Bytes, H256, U256, U64},
+    types::{Address, BlockNumber, Bytes, H160, H256, U256, U64},
     utils::keccak256,
 };
-use serde::{ser::SerializeStruct, Deserialize, Serialize, Serializer};
+use serde::{
+    de::DeserializeOwned, ser::SerializeStruct, Deserialize, Deserializer, Serialize, Serializer,
+};
 use std::ops::{Range, RangeFrom, RangeTo};
 
 /// A log produced by a transaction.
-#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
+#[derive(Debug, Clone, Default, PartialEq, Serialize, Deserialize)]
 pub struct Log {
-    /// H160
+    /// H160. the contract that emitted the log
     pub address: Address,
 
     /// topics: Array of 0 to 4 32 Bytes of indexed log arguments.
@@ -41,7 +43,7 @@ pub struct Log {
     #[serde(skip_serializing_if = "Option::is_none")]
     pub transaction_index: Option<U64>,
 
-    /// Integer of the log index position in the block. Noe if it's a pending log.
+    /// Integer of the log index position in the block. None if it's a pending log.
     #[serde(rename = "logIndex")]
     #[serde(skip_serializing_if = "Option::is_none")]
     pub log_index: Option<U256>,
@@ -58,11 +60,20 @@ pub struct Log {
     pub log_type: Option<String>,
 
     /// True when the log was removed, due to a chain reorganization.
-    /// false if its a valid log.
+    /// false if it's a valid log.
     #[serde(skip_serializing_if = "Option::is_none")]
     pub removed: Option<bool>,
 }
 
+impl rlp::Encodable for Log {
+    fn rlp_append(&self, s: &mut rlp::RlpStream) {
+        s.begin_list(3);
+        s.append(&self.address);
+        s.append_list(&self.topics);
+        s.append(&self.data.0);
+    }
+}
+
 #[derive(Copy, Clone, Debug, PartialEq)]
 pub enum FilterBlockOption {
     Range { from_block: Option<BlockNumber>, to_block: Option<BlockNumber> },
@@ -123,6 +134,7 @@ impl Default for FilterBlockOption {
 }
 
 impl FilterBlockOption {
+    #[must_use]
     pub fn set_from_block(&self, block: BlockNumber) -> Self {
         let to_block =
             if let FilterBlockOption::Range { to_block, .. } = self { *to_block } else { None };
@@ -130,6 +142,7 @@ impl FilterBlockOption {
         FilterBlockOption::Range { from_block: Some(block), to_block }
     }
 
+    #[must_use]
     pub fn set_to_block(&self, block: BlockNumber) -> Self {
         let from_block =
             if let FilterBlockOption::Range { from_block, .. } = self { *from_block } else { None };
@@ -137,6 +150,7 @@ impl FilterBlockOption {
         FilterBlockOption::Range { from_block, to_block: Some(block) }
     }
 
+    #[must_use]
     pub fn set_hash(&self, hash: H256) -> Self {
         FilterBlockOption::AtBlockHash(hash)
     }
@@ -157,9 +171,6 @@ pub struct Filter {
     // TODO: We could improve the low level API here by using ethabi's RawTopicFilter
     // and/or TopicFilter
     pub topics: [Option<ValueOrArray<H256>>; 4],
-
-    /// Limit
-    limit: Option<usize>,
 }
 
 impl Serialize for Filter {
@@ -199,10 +210,6 @@ impl Serialize for Filter {
         }
         s.serialize_field("topics", &filtered_topics)?;
 
-        if let Some(ref limit) = self.limit {
-            s.serialize_field("limit", limit)?;
-        }
-
         s.end()
     }
 }
@@ -273,72 +280,101 @@ impl Filter {
     /// let filter = Filter::new().select(..1337u64);
     /// # }
     /// ```
+    #[must_use]
     pub fn select(mut self, filter: impl Into<FilterBlockOption>) -> Self {
         self.block_option = filter.into();
         self
     }
 
     #[allow(clippy::wrong_self_convention)]
+    #[must_use]
     pub fn from_block<T: Into<BlockNumber>>(mut self, block: T) -> Self {
         self.block_option = self.block_option.set_from_block(block.into());
         self
     }
 
     #[allow(clippy::wrong_self_convention)]
+    #[must_use]
     pub fn to_block<T: Into<BlockNumber>>(mut self, block: T) -> Self {
         self.block_option = self.block_option.set_to_block(block.into());
         self
     }
 
     #[allow(clippy::wrong_self_convention)]
+    #[must_use]
     pub fn at_block_hash<T: Into<H256>>(mut self, hash: T) -> Self {
         self.block_option = self.block_option.set_hash(hash.into());
         self
     }
-
+    /// Sets the inner filter object
+    ///
+    /// *NOTE:* ranges are always inclusive
+    ///
+    /// # Examples
+    ///
+    /// Match only a specific address `("0xAc4b3DacB91461209Ae9d41EC517c2B9Cb1B7DAF")`
+    ///
+    /// ```rust
+    /// # use ethers_core::types::{Filter, Address};
+    /// # fn main() {
+    /// let filter = Filter::new().address("0xAc4b3DacB91461209Ae9d41EC517c2B9Cb1B7DAF".parse::<Address>().unwrap());
+    /// # }
+    /// ```
+    ///
+    /// Match all addresses in array `(vec!["0xAc4b3DacB91461209Ae9d41EC517c2B9Cb1B7DAF",
+    /// "0x8ad599c3A0ff1De082011EFDDc58f1908eb6e6D8"])`
+    ///
+    /// ```rust
+    /// # use ethers_core::types::{Filter, Address, ValueOrArray};
+    /// # fn main() {
+    /// let addresses = vec!["0xAc4b3DacB91461209Ae9d41EC517c2B9Cb1B7DAF".parse::<Address>().unwrap(),"0x8ad599c3A0ff1De082011EFDDc58f1908eb6e6D8".parse::<Address>().unwrap()];
+    /// let filter = Filter::new().address(addresses);
+    /// # }
+    /// ```
+    #[must_use]
     pub fn address<T: Into<ValueOrArray<Address>>>(mut self, address: T) -> Self {
         self.address = Some(address.into());
         self
     }
 
     /// given the event in string form, it hashes it and adds it to the topics to monitor
+    #[must_use]
     pub fn event(self, event_name: &str) -> Self {
         let hash = H256::from(keccak256(event_name.as_bytes()));
         self.topic0(hash)
     }
 
     /// Sets topic0 (the event name for non-anonymous events)
+    #[must_use]
     pub fn topic0<T: Into<ValueOrArray<H256>>>(mut self, topic: T) -> Self {
         self.topics[0] = Some(topic.into());
         self
     }
 
     /// Sets the 1st indexed topic
+    #[must_use]
     pub fn topic1<T: Into<ValueOrArray<H256>>>(mut self, topic: T) -> Self {
         self.topics[1] = Some(topic.into());
         self
     }
 
     /// Sets the 2nd indexed topic
+    #[must_use]
     pub fn topic2<T: Into<ValueOrArray<H256>>>(mut self, topic: T) -> Self {
         self.topics[2] = Some(topic.into());
         self
     }
 
     /// Sets the 3rd indexed topic
+    #[must_use]
     pub fn topic3<T: Into<ValueOrArray<H256>>>(mut self, topic: T) -> Self {
         self.topics[3] = Some(topic.into());
         self
     }
-
-    pub fn limit(mut self, limit: usize) -> Self {
-        self.limit = Some(limit);
-        self
-    }
 }
 
 /// Union type for representing a single value or a vector of values inside a filter
-#[derive(Debug, PartialEq, Clone)]
+#[derive(Debug, PartialEq, Eq, Clone, Hash)]
 pub enum ValueOrArray<T> {
     /// A single value
     Value(T),
@@ -348,6 +384,18 @@ pub enum ValueOrArray<T> {
 
 // TODO: Implement more common types - or adjust this to work with all Tokenizable items
 
+impl From<H160> for ValueOrArray<H160> {
+    fn from(src: H160) -> Self {
+        ValueOrArray::Value(src)
+    }
+}
+
+impl From<Vec<H160>> for ValueOrArray<H160> {
+    fn from(src: Vec<H160>) -> Self {
+        ValueOrArray::Array(src)
+    }
+}
+
 impl From<H256> for ValueOrArray<H256> {
     fn from(src: H256) -> Self {
         ValueOrArray::Value(src)
@@ -385,12 +433,60 @@ where
     }
 }
 
+impl<'a, T> Deserialize<'a> for ValueOrArray<T>
+where
+    T: DeserializeOwned,
+{
+    fn deserialize<D>(deserializer: D) -> Result<ValueOrArray<T>, D::Error>
+    where
+        D: Deserializer<'a>,
+    {
+        let value = serde_json::Value::deserialize(deserializer)?;
+
+        if value.is_null() {
+            return Ok(ValueOrArray::Array(Vec::new()))
+        }
+
+        #[derive(Deserialize)]
+        #[serde(untagged)]
+        enum Variadic<T> {
+            Value(T),
+            Array(Vec<T>),
+        }
+
+        match serde_json::from_value::<Variadic<T>>(value).map_err(|err| {
+            serde::de::Error::custom(format!("Invalid variadic value or array type: {}", err))
+        })? {
+            Variadic::Value(val) => Ok(ValueOrArray::Value(val)),
+            Variadic::Array(arr) => Ok(ValueOrArray::Array(arr)),
+        }
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
     use crate::utils::serialize;
     use serde_json::json;
 
+    #[test]
+    fn can_serde_value_or_array() {
+        #[derive(Clone, Debug, Deserialize, Serialize, PartialEq)]
+        struct Item {
+            value: ValueOrArray<U256>,
+        }
+
+        let item = Item { value: ValueOrArray::Value(U256::one()) };
+        let json = serde_json::to_value(item.clone()).unwrap();
+        let deserialized: Item = serde_json::from_value(json).unwrap();
+        assert_eq!(item, deserialized);
+
+        let item = Item { value: ValueOrArray::Array(vec![U256::one(), U256::zero()]) };
+        let json = serde_json::to_value(item.clone()).unwrap();
+        let deserialized: Item = serde_json::from_value(json).unwrap();
+        assert_eq!(item, deserialized);
+    }
+
     #[test]
     fn filter_serialization_test() {
         let t1 = "9729a6fbefefc8f6005933898b13dc45c3a2c8b7".parse::<Address>().unwrap();
