@@ -56,7 +56,7 @@ pub enum Eip712Error {
 /// for `derive_eip712` for more information and example usage.
 ///
 /// For those who wish to manually implement this trait, see:
-/// https://eips.ethereum.org/EIPS/eip-712
+/// <https://eips.ethereum.org/EIPS/eip-712>
 ///
 /// Any rust struct implementing Eip712 must also have a corresponding
 /// struct in the verifying ethereum contract that matches its signature.
@@ -166,6 +166,7 @@ impl<T: Eip712 + Clone> EIP712WithDomain<T> {
         Ok(Self { domain, inner })
     }
 
+    #[must_use]
     pub fn set_domain(self, domain: EIP712Domain) -> Self {
         Self { domain, inner: self.inner }
     }
@@ -266,16 +267,17 @@ impl TryFrom<&syn::DeriveInput> for EIP712Domain {
                                                         .to_compile_error())
                                                     }
 
-                                                    domain.chain_id = lit_int
-                                                        .base10_digits()
-                                                        .parse()
-                                                        .map_err(|_| {
-                                                            Error::new(
-                                                                meta.path.span(),
-                                                                "failed to parse chain id",
-                                                            )
-                                                            .to_compile_error()
-                                                        })?;
+                                                    domain.chain_id = U256::from(
+                                                        lit_int.base10_parse::<u64>().map_err(
+                                                            |_| {
+                                                                Error::new(
+                                                                    meta.path.span(),
+                                                                    "failed to parse chain id",
+                                                                )
+                                                                .to_compile_error()
+                                                            },
+                                                        )?,
+                                                    );
                                                 }
                                                 _ => {
                                                     return Err(Error::new(
@@ -410,7 +412,7 @@ impl TryFrom<&syn::DeriveInput> for EIP712Domain {
 }
 
 /// Parse the eth abi parameter type based on the syntax type;
-/// this method is copied from https://github.com/gakonst/ethers-rs/blob/master/ethers-contract/ethers-contract-derive/src/lib.rs#L600
+/// this method is copied from <https://github.com/gakonst/ethers-rs/blob/master/ethers-contract/ethers-contract-derive/src/lib.rs#L600>
 /// with additional modifications for finding byte arrays
 pub fn find_parameter_type(ty: &Type) -> Result<ParamType, TokenStream> {
     match ty {
